# -*- coding: utf-8 -*-

class Library(object):
    instance = None

    _globals = {}
    _tests = {}
    _filters = {}

    def __new__(cls, *args, **kwargs):
        if cls.instance == None:
            cls.instance = super(Library, cls).__new__(cls, *args, **kwargs)
        return cls.instance

    @classmethod
    def get_instance(cls):
        return cls.instance

    def _update_env(self, env):
        env.filters.update(self._filters)
        env.globals.update(self._globals)
        env.tests.update(self._tests)

    def _new_function(self, attr, func, name=None):
        _attr = getattr(self, attr)
        if name is None:
            name = func.__name__

        _attr[name] = func
        return func

    def _function(self, attr, name=None, _function=None):
        if name is None and _function is None:
            def dec(func):
                return self._new_function(attr, func)
            return dec

        elif name is not None and _function is None:
            if callable(name):
                return self._new_function(attr, name)

            else:
                def dec(func):
                    return self._function(attr, name, func)

                return dec

        elif name is not None and _function is not None:
            return self._new_function(attr, _function, name)

        raise RuntimeError("Invalid parameters")

    def global_function(self, *args, **kwargs):
        return self._function("_globals", *args, **kwargs)

    def test(self, *args, **kwargs):
        return self._function("_tests", *args, **kwargs)

    def filter(self, *args, **kwargs):
        return self._function("_filters", *args, **kwargs)

    def __setitem__(self, item, value):
        self._globals[item] = value

    def __getitem__(self, item, value): #for reciprocity with __setitem__
        return self._globals[item]



from __future__ import unicode_literals

from django.conf import settings
from django.core.cache import cache
from django.utils.http import urlquote

from jinja2.ext import Extension
from jinja2 import nodes
from jinja2 import Markup

import hashlib
import traceback


class CsrfExtension(Extension):
    tags = set(['csrf_token'])

    def __init__(self, environment):
        self.environment = environment

    def parse(self, parser):
        try:
            token = next(parser.stream)
            call_res = self.call_method('_render', [nodes.Name('csrf_token','load')])
            return nodes.Output([call_res]).set_lineno(token.lineno)
        except Exception:
            traceback.print_exc()

    def _render(self, csrf_token):
        if csrf_token:
            if csrf_token == 'NOTPROVIDED':
                return Markup("")

            return Markup("<input type='hidden'"
                          " name='csrfmiddlewaretoken' value='%s' />" % (csrf_token))

        if settings.DEBUG:
            import warnings
            warnings.warn("A {% csrf_token %} was used in a template, but the context"
                          "did not provide the value.  This is usually caused by not "
                          "using RequestContext.")
        return ''


class CacheExtension(Extension):
    """
    Exactly like Django's own tag, but supports full Jinja2
    expressiveness for all arguments.

        {% cache gettimeout()*2 "foo"+options.cachename  %}
            ...
        {% endcache %}

    General Syntax:

        {% cache [expire_time] [fragment_name] [var1] [var2] .. %}
            .. some expensive processing ..
        {% endcache %}

    Available by default (does not need to be loaded).

    Partly based on the ``FragmentCacheExtension`` from the Jinja2 docs.
    """

    tags = set(['cache'])

    def parse(self, parser):
        lineno = next(parser.stream).lineno

        expire_time = parser.parse_expression()
        fragment_name = parser.parse_expression()
        vary_on = []

        while not parser.stream.current.test('block_end'):
            vary_on.append(parser.parse_expression())

        body = parser.parse_statements(['name:endcache'], drop_needle=True)

        return nodes.CallBlock(
            self.call_method('_cache_support',
                             [expire_time, fragment_name,
                              nodes.List(vary_on), nodes.Const(lineno)]),
            [], [], body).set_lineno(lineno)

    def _cache_support(self, expire_time, fragm_name, vary_on, lineno, caller):
        try:
            expire_time = int(expire_time)
        except (ValueError, TypeError):
            raise TemplateSyntaxError('"%s" tag got a non-integer timeout '
                'value: %r' % (list(self.tags)[0], expire_time), lineno)

        args_map = map(urlquote, vary_on)
        args_map = map(lambda x: x.encode('utf-8'), args_map)

        args_string = b':'.join(args_map)
        args_hash = hashlib.md5(args_string).hexdigest()

        cache_key = 'template.cache.{0}.{1}'.format(fragm_name, args_hash)

        value = cache.get(cache_key)
        if value is not None:
            return value.decode('utf-8')

        value = caller()
        cache.set(cache_key, value.encode('utf-8'), expire_time)
        return value



# -*- coding: utf-8 -*-

from django.core.urlresolvers import reverse as django_reverse
from django.contrib.staticfiles.storage import staticfiles_storage

def url(name, *args, **kwargs):
    """
    Shortcut filter for reverse url on templates. Is a alternative to
    django {% url %} tag, but more simple.

    Usage example:
        {{ url('web:timeline', userid=2) }}

    This is a equivalent to django:
        {% url 'web:timeline' userid=2 %}

    """
    return django_reverse(name, args=args, kwargs=kwargs)


def static(path):
    return staticfiles_storage.url(path)



# -*- coding: utf-8 -*-

from django.views.generic import View
from django.template import loader, RequestContext
from django import http


class GenericView(View):
    response_cls = http.HttpResponse
    content_type = "text/html"
    tmpl_name = None

    def get_context_data(self):
        return {"view": self}

    def get(self, request, *args, **kwargs):
        context = self.get_context_data()
        output = loader.render_to_string(self.tmpl_name, context,
                                         context_instance=RequestContext(request))

        return self.response_cls(output, content_type=self.content_type)


class PageNotFound(GenericView):
    tmpl_name = "404.jinja"
    response_cls = http.HttpResponseNotFound


class PermissionDenied(GenericView):
    tmpl_name = "403.jinja"
    response_cls = http.HttpResponseForbidden


class ServerError(GenericView):
    tmpl_name = "500.jinja"
    response_cls = http.HttpResponseServerError



# -*- coding: utf-8 -*-

from easy_thumbnails.templatetags import thumbnail as _thumbnail
from django_jinja import library
lib = library.Library()


@lib.filter
def thumbnail_url(source, alias):
    return _thumbnail.thumbnail_url(source, alias)


@lib.global_function
def thumbnailer_passive(obj):
    return _thumbnail.thumbnailer_passive(obj)


@lib.global_function
def thumbnailer(obj):
    return _thumbnail.thumbnailer(obj)


@lib.global_function
def thumbnail(source, **kwargs):
    thumb = _thumbnail.get_thumbnailer(source).get_thumbnail(kwargs)
    return thumb.url
